/* This is the first part of the SyntheSys system manger. The other part is in libSystem.

It contains C boot routines and system base routines.

*/

#include <libSystem/TermPCCons/TermPCCons.h>
#include "SystemMgr-ck.h"
#include <SysCore/SC_mem.h>
#include <libSystem/InputMgr/KeyboardBase.h>
#include <CoreKernel/TimeMgr/TimeMgr.h>
#define sysGDTSize	32

struct SSSysGDTEntry_s SysGDTTable[sysGDTSize];
struct SSSysGDTPtr_s SystemGDT;
struct SSSysIDTEntry_s SysIDTTable[256];
struct SSSysIDTPtr_s SystemIDT;

unsigned char *SysExceptionMsg[] = 
{
"Division By Zero Exception\n",
"Debug Exception\n",
"Non Maskable Interrupt Exception\n",
"Breakpoint Exception\n",
"Into Detected Overflow Exception\n",
"Out of Bounds Exception\n",
"Invalid Opcode Exception\n",
"No Coprocessor Exception\n",
"Double Fault Exception\n",
"Coprocessor Segment Overrun Exception\n",
"Bad TSS Exception (Should not happen, as SyntheSys is using software based task switching)\n",
"Segment Not Present Exception\n",
"Stack Fault Exception\n",
"General Protection Fault Exception\n",
"Page Fault Exception\n",
"Unknown Interrupt Exception\n",
"Coprocessor Fault Exception\n",
"Alignment Check Exception (486+)\n",
"Machine Check Exception (Pentium/586+)\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
"Reserved Exception\n",
};


void SysBoot(unsigned long MultibootMagic, unsigned long MultiBootInfoAddr) {
TermConsInit();
TermTextPrint("SyntheSys CoreKernel v0.1alpha booting ...\n\n",WHITE,BLACK);

SSMultibootSysInfo_s *MultiBootInfoBlock;

MultiBootInfoBlock = (unsigned long)MultiBootInfoAddr;

if (MultibootMagic != 0x2BADB002) {
	TermTextPrint("* System manager: Wrong multiboot magic number\n",WHITE,BLACK);
}

TermTextPrint("* System manager: Installing GDT\n",WHITE,BLACK);
SysGDTInstall();
TermTextPrint("* System manager: Installing IDT\n",WHITE,BLACK);
SysIDTInstall();
TermTextPrint("* System manager: Installing main ISRs\n",WHITE,BLACK);
SysISRInstall();
TermTextPrint("* System manager: Installing IRQs handler\n",WHITE,BLACK);
SysIRQInstall();
TimerIRQInstall();
InputKbdIRQInstall();
TermTextPrint("* System manager: Allowing IRQs to happen\n",WHITE,BLACK);
__asm__ __volatile__ ("sti");
TermTextPrint("* System manager: Acquiring multiboot system information\n",WHITE,BLACK);
MultiBootInfoBlock = (unsigned long)MultiBootInfoAddr;

TermTextPrint("\n* System manager: nanoSys booted up !\n\n",WHITE,BLACK);

}

void SysHalt() {
TermTextPrint("* System manager: System halted, you can securely close your computer now\n",WHITE,BLACK);
for (;;);
}

void SysGDTGateSet(int Num, unsigned long Base, unsigned long Limit, unsigned char AccessPrivileges, unsigned char BlockSize) {

    SysGDTTable[Num].BaseLow = (Base & 0xFFFF);
    SysGDTTable[Num].BaseMiddle = (Base >> 16) & 0xFF;
    SysGDTTable[Num].BaseHigh = (Base >> 24) & 0xFF;

    SysGDTTable[Num].LimitLow = (Limit & 0xFFFF);
    SysGDTTable[Num].BlockSize = ((Limit >> 16) & 0x0F);

    SysGDTTable[Num].BlockSize |= (BlockSize & 0xF0);
    SysGDTTable[Num].AccessPrivilieges = AccessPrivileges;

}

void SysGDTInstall(void) {

    SystemGDT.Limit = (sizeof(struct SSSysGDTEntry_s) * sysGDTSize) - 1;
    SystemGDT.Base = &SysGDTTable;

    /* Our NULL descriptor (required by the cpu) */
    SysGDTGateSet(0, 0, 0, 0, 0);

    /* The second entry is our Code Segment. The base address
    *  is 0, the limit is 4GBytes, it uses 4KByte granularity,
    *  uses 32-bit opcodes, and is a Code Segment descriptor.
    *  Please check the table above in the tutorial in order
    *  to see exactly what each value means */

    SysGDTGateSet(1, 0, 0xFFFFFFFF, 0x9A, 0xCF);

    /* The third entry is our Data Segment. It's EXACTLY the
    *  same as our code segment, but the descriptor type in
    *  this entry's access byte says it's a Data Segment */
    SysGDTGateSet(2, 0, 0xFFFFFFFF, 0x92, 0xCF);

    /* Flush out the old GDT and install the new changes! */
    SysGDTUpdate();
}

void SysIDTEntrySet(int Num, unsigned long Base, unsigned short SegmentSelector, unsigned char MaxRingCallable) {

	SysIDTTable[Num].BaseLo = (Base & 0xFFFF);
	SysIDTTable[Num].BaseHi = (Base >> 16) & 0xFFFF;

	SysIDTTable[Num].SegmentSelector = SegmentSelector;
	SysIDTTable[Num].Always0 = 0;

	SysIDTTable[Num].AccessFlags = 14 + (MaxRingCallable * 32) + 128;
}

void SysIDTInstall(void) {

SystemIDT.Limit = (sizeof(struct SSSysGDTPtr_s) * 256) - 1;
SystemIDT.Base = &SysIDTTable;

SC_MemSet(&SysIDTTable,(sizeof(struct SSSysGDTPtr_s) * 256),0);
SysIDTLoad();
}

void SysISRHandler(struct SSSysRegs_s *regsp) {
if (regsp->int_no < 32) {
	TermTextPrint(SysExceptionMsg[regsp->int_no],WHITE,BLACK);
	TermTextPrint("* System manager: Exception has occured, halting system\n",WHITE,BLACK);
	SysHalt();
}
}

void SysISRInstall(void) {
    
    SysIDTEntrySet(0, (unsigned)SysISR0, 0x08, 0);
    SysIDTEntrySet(1, (unsigned)SysISR1, 0x08, 0);
    SysIDTEntrySet(2, (unsigned)SysISR2, 0x08, 0);
    SysIDTEntrySet(3, (unsigned)SysISR3, 0x08, 0);
    SysIDTEntrySet(4, (unsigned)SysISR4, 0x08, 0);
    SysIDTEntrySet(5, (unsigned)SysISR5, 0x08, 0);
    SysIDTEntrySet(6, (unsigned)SysISR6, 0x08, 0);
    SysIDTEntrySet(7, (unsigned)SysISR7, 0x08, 0);
    SysIDTEntrySet(8, (unsigned)SysISR8, 0x08, 0);
    SysIDTEntrySet(9, (unsigned)SysISR9, 0x08, 0);
    SysIDTEntrySet(10, (unsigned)SysISR10, 0x08, 0);
    SysIDTEntrySet(11, (unsigned)SysISR11, 0x08, 0);
    SysIDTEntrySet(12, (unsigned)SysISR12, 0x08, 0);
    SysIDTEntrySet(13, (unsigned)SysISR13, 0x08, 0);
    SysIDTEntrySet(14, (unsigned)SysISR14, 0x08, 0);
    SysIDTEntrySet(15, (unsigned)SysISR15, 0x08, 0);
    SysIDTEntrySet(16, (unsigned)SysISR16, 0x08, 0);
    SysIDTEntrySet(17, (unsigned)SysISR17, 0x08, 0);
    SysIDTEntrySet(18, (unsigned)SysISR18, 0x08, 0);
    SysIDTEntrySet(19, (unsigned)SysISR19, 0x08, 0);
    SysIDTEntrySet(20, (unsigned)SysISR20, 0x08, 0);
    SysIDTEntrySet(21, (unsigned)SysISR21, 0x08, 0);
    SysIDTEntrySet(22, (unsigned)SysISR22, 0x08, 0);
    SysIDTEntrySet(23, (unsigned)SysISR23, 0x08, 0);
    SysIDTEntrySet(24, (unsigned)SysISR24, 0x08, 0);
    SysIDTEntrySet(25, (unsigned)SysISR25, 0x08, 0);
    SysIDTEntrySet(26, (unsigned)SysISR26, 0x08, 0);
    SysIDTEntrySet(27, (unsigned)SysISR27, 0x08, 0);
    SysIDTEntrySet(28, (unsigned)SysISR28, 0x08, 0);
    SysIDTEntrySet(29, (unsigned)SysISR29, 0x08, 0);
    SysIDTEntrySet(30, (unsigned)SysISR30, 0x08, 0);
    SysIDTEntrySet(31, (unsigned)SysISR31, 0x08, 0);

}

