/* TermPCCons.c
	
	This file contains the code to print text and manage a 
	standard VGA console. As a lot of parts of SyntheSys, it uses SysCore to do interrupts
	and read the return values.
	
	Functions & procedures:
	int TermConsInit(void) V
	int TermTextPrint(char *text) V
	int TermCellGetAddress(unsigned char row,unsigned char column) V
	int TermCellGetAttrAddress(unsigned char row,unsigned char column) V
	int TermCellGetPositionByAddress(unsigned long addr) V
	int TermCursorSetAtXY(unsigned char row,unsigned char column)
	int TermCursorSetAtAddress(unsigned long addr)
	
*/

#include <SysCore/SC_mem.h>
#include <SysCore/SC_io.h>
#include "TermPCCons.h"

ConsCursorPos_s curpos; // Current cursor position
const unsigned int ConsSize = 80*25;
unsigned char *fb; // The text framebuffer base address
unsigned long fbstart;
 
/* 
	TermConsInit
	Initializes the standard console.
*/

int TermConsInit(void) {
unsigned int *Addr6845 = (unsigned int *)0x463;
fb = (unsigned char *)((*Addr6845 == 0x3B4) ? 0xB0000 : 0xB8000);
fbstart = (unsigned long)fb;
TermScreenClear();
return 0;
}

int TermTextPrint(char *text, unsigned char fgcolor, unsigned char bgcolor) {
unsigned char attribute;
unsigned int i;
ConsCursorPos_s pcpos;

if (curpos.y == 24) {
	TermScroll(1);
	curpos.x = 0;
	curpos.y = 23;
}

fb = (unsigned char *)TermCellGetAddress(curpos.x, curpos.y);
attribute = (bgcolor << 4) | fgcolor;

for (i=0; *text; i += 2) {
	if (*text < ' ') {
		switch (*text) {
			case '\n':
				curpos.y++;
				curpos.x = 0;	
				fb = (unsigned char *)TermCellGetAddress(curpos.x, curpos.y);
				text++;
				break;
			case '\t':
				curpos.x = (curpos.x + 8) & ~(8 - 1);
				fb = (unsigned char *)TermCellGetAddress(curpos.x, curpos.y);
				text++;
				break;
			case 0x08:
				curpos.x--;
				fb = (unsigned char *)TermCellGetAddress(curpos.x, curpos.y);
				text++;
				break;
		 }
	}
	else {
		*fb = *text;
		fb++;
		*fb = attribute;
		fb++;
		text++;
	}
	TermCellGetPosByAddr((unsigned long)fb,&pcpos);
	TermCursorSetAtXY(0,(pcpos.y++));
}

return 0;
}

void TermCharPut(unsigned char CharToPut, unsigned char fgcolor, unsigned char bgcolor) {
ConsCursorPos_s pcpos;
unsigned char attribute;

fb = (unsigned char *)TermCellGetAddress(curpos.x, curpos.y);
attribute = (bgcolor << 4) | fgcolor;
*fb = CharToPut;
fb++;
*fb = attribute;
fb++;
TermCellGetPosByAddr((unsigned long)fb,&pcpos);
TermCursorSetAtXY(0,(pcpos.y++));
}

unsigned long TermCellGetAddress(unsigned char row, unsigned char column) {
fb = (unsigned char *)fbstart;
return ((unsigned long)fb) + ((column * 160) + (row * 2)); // We get the row base address by multiplying 'row' by 160 and adding the column * 2 this gets us an addr to a char cell of the fb.
}

unsigned long TermCellGetAttrAddress(unsigned char row, unsigned char column) {
return (TermCellGetAddress(row,column)+1);
}

int TermScroll(unsigned int lines) {
fb = (unsigned char *)fbstart;
SC_MemDataMove(fb + (lines * 160), fb, (ConsSize*2) - (lines * 160));
SC_MemSet((void *)TermCellGetAddress(0,(25-lines)),(lines * 160),0);
return (curpos.y - lines);
}


int TermCellGetPosByAddr(unsigned long addr, ConsCursorPos_s *addrcurs) {
unsigned long xpos,ypos,tmp;

tmp = addr -fbstart;

xpos = (tmp % 160) / 2;
ypos = (tmp - (xpos * 2)) / 160;

addrcurs->x = xpos;
addrcurs->y = ypos;
return addr;
}

void TermScreenClear(void) {
unsigned char attr;
unsigned int i;

attr = (BLACK << 4) | WHITE;
fb = (unsigned char *)fbstart;

for (i=0;(i <= 2000);i++) {
	*fb = 0x20;
	fb++;
	*fb = attr;
	fb++;
}
fb = (unsigned char *)fbstart;
TermCursorSetAtXY(0,0);
}

void TermCursorSetAtXY(unsigned char wx,unsigned char wy) {
unsigned hwaddr;

/* Setting our virtual cursor */
curpos.x = wx;
curpos.y = wy;

/* Setting the hardware cursor */
hwaddr = (wy * 80) + wx;
SC_IOPortWrite(0x3D4,14);
SC_IOPortWrite(0x3D5, hwaddr >> 8);
SC_IOPortWrite(0x3D4,15);
SC_IOPortWrite(0x3D5,hwaddr);
}
